******************************************************
** PROJECT     : TORRENT
** TITLE       : chiron sequencer prototype (Slow readout @ 2us dwell).
** VERSION     : 1.4
** HISTORY     : VER    DATE     AUTHOR     COMMENTS
**             : 1.0    12/30/11  PCM    Adapting from chiron_sequencer_fast.asm version 1.0 code
**             : 1.1    01/17/12  PCM    Runnable assembler giving approx. 9e-rm on four quadrants
**             : 1.2    01/30/12  PCM    Normalized binning mode to not use the binning flag
**             : 1.3    02/03/12  PCM    Tweaking for noise / speed performance. Achieved 7 ADU rms.
**             : 1.4    02/17/12  PCM    Removed the call to clear before the opening shutter and enabled the shutter enable user bit for the mountain 
**             : 1.5    02/21/12  PCM    Corrected binning code ... My bad :-(
** DESCRIPTION : This code is used to run the E2V231-84 Chiron detector on the Torrent DHE hardware.
**             : at the 1.5Mtr Tololo with 2 or 4 amplifier readout. 
**             :
**             : Compile as "tasm chiron_torrent_sequencer_slow_rev1.5.asm chiron_sequencer_slow.ucd"
**             :
**
******************************************************
** SIGNAL ASSIGNMENTS :
**# Configuration Document: TRNT-AD-12-0209_CfgRpt_chiron.txt for System chiron
**# Configuration Report for System ID=chiron
**# sysConfig Version 1.05 of Wednesday 20111102:1228
**#
**# System Description Last Saved 20111207:1239
**#
**# Responsible Engineer: Peter Moore
**# System User: CTIO
**#
**# Report Printed: 20111216:0842
**#
**##############################################################################
** The chiron system documentation number is 209
** Transition Module Number: ???
** Controller Module Number: ???
**##############################################################################
**#
**# chiron Focal Plane Description
**#
**# chiron built using fclplnDesc Version 1.03 of Wednesday20110713.1315
**# Date last saved: 20111207:1239
**#
** The chiron focal plane contains 1 detector
** The Detector is a e2v231-84 with 4096 columns by 4112 rows: ID = ccd1
**##############################################################################
**#
**# chiron Dewar Description
**#
**# chiron built using dewarDesc Version 1.04 of Wednesday 20110713.1655:46
**# Date last saved: 20111130:0857
**#
** The chiron Dewar has 2 connectors
** Connector 1 is P3: a MILCIRC_I with 41 pins
** Connector 2 is P4: a MILCIRC_I with 41 pins
**##############################################################################
**#
**# generic2AFE Torrent DHE Description
**#
**# generic2AFE built using TRNT dheDesc Version 1.05 of Thursday 20110714.1010
**# Date last Saved: 20111207:1239
**#
** The NONE DHE contains 7 Boards:
** LCB Board LCB Serial Number:9243 Last Test:20100301
** PSB Board PSB Serial Number:9234 Last Test:20091202
** AFE Board AFE1 Serial Number:9357 Last Test:20091202
** AFE Board AFE2 Serial Number:9358 Last Test:20091202
** MEZ Board MEZZ Serial Number:9914 Last Test:20110712
** PAMP Board PreAmp Serial Number:9123 Last Test:20110712
** TSMU Board TSMUtil Serial Number:9910 Last Test:20110712
** Torrent DHE adjustable voltage settings: HV volts select = +30v to -5v
** vHV+ = 30.000 vHV- = -5.000
** vCB+ = 17.500 vCB- = -17.500
** vAna+ = 11.000 vAna- = -11.000
** vBb = 0.000
** 
**# Configuration Document: TRNT-EL-05-0209_WireList for System chiron
**# sysConfig Version 1.05 of Wednesday 20111102:1228
**#
**# System Description Last Saved 20111207:1239
**#
**# WireList for system chiron
**# Report Printed 20111216:0842
**#
**# Responsible Engineer: Peter Moore
**# System User: CTIO
**#
**# Use 24 gauge Teflon wires in standard Torrent colors
**# Clocks - white, Clock Rtns - white/black
**# LV Biases - yellow LV Bias Rtns - black
**# HV biases - orange HV Bias Rtns - black
**# Video signals - violet Video Rtns - white/violet
**# Aux signals - grey, Aux Rtns - white/grey
**# Heater - blue, Heater Rtn - white/blue
**# TC+ - blue copper, TC- - red constantin
**#
**#CNCT Det Sgnl - Array Pins => Dewar Pins => Dhe Pins :Dhe Function Name
** CNCT SUB0 - ccd1:C1P01 => P3:s => Video2:CH3-A :afe2Vid03Rtn
** CNCT OS-E - ccd1:C1P03 => P3:n => Video2:CH1+A :afe2vChnl01
** CNCT OG-E - ccd1:C1P04 => P3:R => J5:01 :afe2LVBias00
** CNCT DG-A - ccd1:C1P05 => P3:T => J3:16 :afe2Clk10
** CNCT RG-E - ccd1:C1P06 => P3:i => J3:36 :afe2Clk00
** CNCT SW-E - ccd1:C1P07 => P3:X => J3:32 :afe2Clk02
** CNCT E1 - ccd1:C1P08 => P3:V => J3:28 :afe2Clk04
** CNCT E2 - ccd1:C1P09 => P3:W => J3:24 :afe2Clk06
** CNCT E3-F3 - ccd1:C1P10 => P3:A => J3:20 :afe2Clk08
** CNCT F1 - ccd1:C1P11 => P3:B => J3:26 :afe2Clk05
** CNCT F2 - ccd1:C1P12 => P3:C => J3:22 :afe2Clk07
** CNCT SW-F - ccd1:C1P13 => P3:Y => J3:30 :afe2Clk03
** CNCT RG-F - ccd1:C1P14 => P3:j => J3:34 :afe2Clk01
** CNCT TG-A - ccd1:C1P15 => P3:Z => J3:18 :afe2Clk09
** CNCT OG-F - ccd1:C1P16 => P3:S => J5:05 :afe2LVBias02
** CNCT OS-F - ccd1:C1P17 => P3:r => Video2:CH3+A :afe2vChnl03
** CNCT SUB1 - ccd1:C1P19 => P3:m => Video2:CH1-A :afe2Vid01Rtn
** CNCT RD-E - ccd1:C1P21 => P3:N => J4:13 :afe2HVBias04
** CNCT OD-E - ccd1:C1P22 => P3:p => J4:01 :afe2HVBias00
** CNCT SUB2 - ccd1:C1P23,ccd1:C1P34 => P3:t => J3:40 :afe2ClkRtn0A
** CNCT A4 - ccd1:C1P24,ccd1:C1P26 => P3:G => J3:06 :afe2Clk15
** CNCT A3 - ccd1:C1P25,ccd1:C1P27 => P3:F => J3:08 :afe2Clk14
** CNCT SUB3 - ccd1:C1P28 => P3:k => J4:03 :afe2HVBiasRtn0
** CNCT DD-A - ccd1:C1P29 => P3:U => J4:22 :afe2HVBias07
** CNCT B1 - ccd1:C1P30,ccd1:C1P32 => P3:D => J3:12 :afe2Clk12
** CNCT A2 - ccd1:C1P33,ccd1:C1P31 => P3:E => J3:10 :afe2Clk13
** CNCT OD-F - ccd1:C1P35 => P3:q => J4:07 :afe2HVBias02
** CNCT RD-F - ccd1:C1P36 => P3:P => J4:19 :afe2HVBias06
** CNCT SUB5 - ccd1:C2P01 => P4:s => Video1:CH3-A :afe1Vid03Rtn
** CNCT OS-G - ccd1:C2P03 => P4:r => Video1:CH3+A :afe1vChnl03
** CNCT OG-G - ccd1:C2P04 => P4:S => J8:05 :afe1LVBias02
** CNCT DG-D - ccd1:C2P05 => P4:T => J6:16 :afe1Clk10
** CNCT RG-G - ccd1:C2P06 => P4:j => J6:34 :afe1Clk01
** CNCT SW-G - ccd1:C2P07 => P4:Y => J6:30 :afe1Clk03
** CNCT G1 - ccd1:C2P08 => P4:B => J6:26 :afe1Clk05
** CNCT G2 - ccd1:C2P09 => P4:C => J6:22 :afe1Clk07
** CNCT G3-H3 - ccd1:C2P10 => P4:A => J6:20 :afe1Clk08
** CNCT H1 - ccd1:C2P11 => P4:V => J6:28 :afe1Clk04
** CNCT H2 - ccd1:C2P12 => P4:W => J6:24 :afe1Clk06
** CNCT SW-H - ccd1:C2P13 => P4:X => J6:32 :afe1Clk02
** CNCT RG-H - ccd1:C2P14 => P4:i => J6:36 :afe1Clk00
**#
**# Use 24 gauge Teflon wires in standard Torrent colors
**# Clocks - white, Clock Rtns - white/black
**# LV Biases - yellow LV Bias Rtns - black
**# HV biases - orange HV Bias Rtns - black
**# Video signals - violet Video Rtns - white/violet
**# Aux signals - grey, Aux Rtns - white/grey
**# Heater - blue, Heater Rtn - white/blue
**# TC+ - blue copper, TC- - red constantin
**#
**#CNCT Det Sgnl - Array Pins => Dewar Pins => Dhe Pins :Dhe Function Name
** CNCT TG-D - ccd1:C2P15 => P4:Z => J6:18 :afe1Clk09
** CNCT OG-H - ccd1:C2P16 => P4:R => J8:01 :afe1LVBias00
** CNCT OS-H - ccd1:C2P17 => P4:n => Video1:CH1+A :afe1vChnl01
** CNCT SUB6 - ccd1:C2P19 => P4:m => Video1:CH1-A :afe1Vid01Rtn
** CNCT RD-G - ccd1:C2P21 => P4:P => J7:19 :afe1HVBias06
** CNCT OD-G - ccd1:C2P22 => P4:q => J7:07 :afe1HVBias02
** CNCT SUB7 - ccd1:C2P23,ccd1:C2P34 => P4:t => J6:40 :afe1ClkRtn0
** CNCT C1 - ccd1:C2P26,ccd1:C2P24 => P4:D => J6:12 :afe1Clk12
** CNCT C2 - ccd1:C2P27,ccd1:C2P25 => P4:E => J6:10 :afe1Clk13
** CNCT SUB8 - ccd1:C2P28 => P4:k => J7:03 :afe1HVBiasRtn0
** CNCT DD-D - ccd1:C2P29 => P4:U => J7:22 :afe1HVBias07
** CNCT C4 - ccd1:C2P30,ccd1:C2P32 => P4:G => J6:06 :afe1Clk15
** CNCT C3 - ccd1:C2P31,ccd1:C2P33 => P4:F => J6:08 :afe1Clk14
** CNCT OD-H - ccd1:C2P35 => P4:p => J7:01 :afe1HVBias00
** CNCT RD-H - ccd1:C2P36 => P4:N => J7:13 :afe1HVBias04
** CNCT vHtr1+ - dfCnct:11 => P4:K => Util-E:E12 :VHTR_P
** CNCT vHtr1Rtn - dfCnct:12 => P4:L => Util-E:E14 :VHTR_N
** CNCT TS1_v+ - dfCnct:2 => P4:H => Util-E:E3 :TEMPSNS1-
** CNCT TS1_v- - dfCnct:4 => P4:J => Util-E:E4 :TEMPSNS1+
** CNCT CableShld1 - dfCnct:5 => P4:M => Util-E:E8 :INPWR_SHIELD
** CNCT TS2_v+ - dfCnct:7 => P4:d => Util-E:E1 :TEMPSNS2-
** CNCT TS2_v- - dfCnct:9 => P4:e => Util-E:E2 :TEMPSNS2+
**#########################################################################
**# grounding for unsed video channels
** Add jumper from Video1*CH2+A: to Video1*CH2-A
** Add jumper from Video1*CH4+A: to Video1*CH4-A
** Add jumper from Video2*CH2+A: to Video2*CH2-A
** Add jumper from Video2*CH4+A: to Video2*CH4-A
**
**# TSM Configuration for system chiron
**#
**# TSM PreAmp Configuration information
**#
**# High voltage Bias protection Diodes for Unspecified detector type
** Install Diode D1 with unknown orientation
** Install Diode D2 with unknown orientation
** Install Diode D3 with unknown orientation
** Install Diode D4 with unknown orientation
** Install Diode D5 with unknown orientation
** Install Diode D6 with unknown orientation
**# Ground connections on PreAmp.
** Preamp Shield-1 (near AFE1 CH4-) connect = none
** Preamp Shield-2 (near AFE2 CH4-) connect = none
** Preamp CH Gnd (near SW1) connect = none
**# TSM Utility Bd Configuration information
**#
**# Temperature Sensor configuration
** Use temperature Sensor 2 for Focalplane temperature control
** Temperature Sensor One is a Two Wire Diode configure as follows:
** Install size 0805 resistor R44 (0 ohms)
** Install size 0805 resistor R42 (0 ohms)
** Temperature Sensor Two is a Two Wire Diode configure as follows:
** Install size 0805 resistor R40 (0 ohms)
** Install size 0805 resistor R38 (0 ohms)
**# Dewar heater configuration
** Using a 25.0 Ohm internal Dewar heater resistor
** Connect J1:E to Util-E:E12 and Connect J1:G Util-E:E13
** or Connect J1:H to Util-E:E12 and Connect J1:K Util-E:E13
** Configure heater current to 0.239 Amps by installing jumper JP3
**# TSM Utility Cables configuration
** No Backside bias usage specified
** Connect Preflash/Shutter cable between Utility board and Lemo connector
** Build external Preflash/Shutter cable to drawing:
**# TSM Grounding configuration
** Utility Bd Shield (E8) connect none
** Utility Bd Chassis Grnd (CHS_GND) connect none
**
**#
**# Defaults from chiron.ini file
**## Defaults from chiron_SetVoltages.mod file
**## Defaults for chiron_DefaultSetup.mod file
**
**
**
**     NOTE:	  each E2V CCD231 serial register has a common S1 S3 S2 sequence for readout on four amplifiers
**               each parallel register has a common P1, p2, p3, p4 sequence for readout on upper and lower halves
**
**             AFE1 : CLK_OUT_00 => RG_H
**             AFE1 : CLK_OUT_01 => RG_G
**             AFE1 : CLK_OUT_02 => SW_H
**             AFE1 : CLK_OUT_03 => SW_G
**             AFE1 : CLK_OUT_04 => S1_H
**             AFE1 : CLK_OUT_05 => S1_G
**             AFE1 : CLK_OUT_06 => S2_H
**             AFE1 : CLK_OUT_07 => S2_G
**             AFE1 : CLK_OUT_08 => S3_GH
**             AFE1 : CLK_OUT_09 => TG_D
**             AFE1 : CLK_OUT_10 => DG_D
**             AFE1 : CLK_OUT_11 => NOT USED
**             AFE1 : CLK_OUT_12 => P1_CD
**             AFE1 : CLK_OUT_13 => P2_CD
**             AFE1 : CLK_OUT_14 => P3_CD
**             AFE1 : CLK_OUT_15 => P4_CD
**
**             AFE2 : CLK_OUT_00 => RG_E
**             AFE2 : CLK_OUT_01 => RG_F
**             AFE2 : CLK_OUT_02 => SW_E
**             AFE2 : CLK_OUT_03 => SW_F
**             AFE2 : CLK_OUT_04 => S1_E
**             AFE2 : CLK_OUT_05 => S1_F
**             AFE2 : CLK_OUT_06 => S2_E
**             AFE2 : CLK_OUT_07 => S2_F
**             AFE2 : CLK_OUT_08 => S3_EF
**             AFE2 : CLK_OUT_09 => TG_A
**             AFE2 : CLK_OUT_10 => DG_A
**             AFE2 : CLK_OUT_11 => NOT USED
**             AFE2 : CLK_OUT_12 => P1_AB
**             AFE2 : CLK_OUT_13 => P2_AB
**             AFE2 : CLK_OUT_14 => P3_AB
**             AFE2 : CLK_OUT_15 => P4_AB
**
***************************
** CLOCK BIT DEFINITIONS ** from Chiron with E2V 231-84 Detector
***************************
DEF #RGh  B00000000000000000000000000000001
DEF #RGg  B00000000000000000000000000000010
DEF #SWh  B00000000000000000000000000000100
DEF #SWg  B00000000000000000000000000001000
DEF #S1h  B00000000000000000000000000010000
DEF #S1g  B00000000000000000000000000100000
DEF #S2h  B00000000000000000000000001000000
DEF #S2g  B00000000000000000000000010000000
DEF #S3gh B00000000000000000000000100000000
DEF #TGcd B00000000000000000000001000000000
DEF #DGcd B00000000000000000000010000000000
DEF #SPcd B00000000000000000000100000000000
DEF #P1cd B00000000000000000001000000000000
DEF #P2cd B00000000000000000010000000000000
DEF #P3cd B00000000000000000100000000000000
DEF #P4cd B00000000000000001000000000000000
DEF #RGe  B00000000000000010000000000000000
DEF #RGf  B00000000000000100000000000000000
DEF #SWe  B00000000000001000000000000000000
DEF #SWf  B00000000000010000000000000000000
DEF #S1e  B00000000000100000000000000000000
DEF #S1f  B00000000001000000000000000000000
DEF #S2e  B00000000010000000000000000000000
DEF #S2f  B00000000100000000000000000000000
DEF #S3ef B00000001000000000000000000000000
DEF #TGab B00000010000000000000000000000000
DEF #DGab B00000100000000000000000000000000
DEF #SPab B00001000000000000000000000000000
DEF #P1ab B00010000000000000000000000000000
DEF #P2ab B00100000000000000000000000000000
DEF #P3ab B01000000000000000000000000000000
DEF #P4ab B10000000000000000000000000000000
*
*********************************
** PARALLEL CLOCK BIT PATTERNS **
*********************************
* Four amplifier mode - amps EFGH
DEF #P1     #P1ab or #P1cd    
DEF #P2     #P2ab or #P2cd
DEF #P3     #P3ab or #P3cd    
DEF #P4     #P4ab or #P4cd

** ELEMENTAL PATTERNS TO MOVE CHARGE ONE PIXEL TOWARDS SERIAL REGISTER
DEF #P_PHASE_0     #P3 or #P2         * Integration contained under phases 2 and 3
DEF #P_PHASE_1     #P3 or #P2 or #P4  * Charge moved under phases 2, 3, and 4
DEF #P_PHASE_2     #P3 or #P4         * Charge contained under phases 3 and 4
DEF #P_PHASE_3     #P3 or #P4 or #P1  * Charge moved under phases 3, 4, and 1
DEF #P_PHASE_4     #P4 or #P1         * Charge contained under phases 4 and 1
DEF #P_PHASE_5     #P4 or #P1 or #P2  * Charge moved under phases 4, 1, and 2
DEF #P_PHASE_6     #P1 or #P2         * Charge contained under phases 1 and 2
DEF #P_PHASE_7     #P1 or #P2 or #P3  * Charge moved under phases 1, 2, and 3
DEF #P_PHASE_8     #P2 or #P3         * Charge contained under phases 2 and 3 -- Copy of state 0 i.e. integration phase

** SHIFT CHARGE TO SERIAL REGISTER
DEF #P_STATE_0     #P_PHASE_0                   * Integration contained under phases 2 and 3
DEF #P_STATE_1     #P_PHASE_1 or #TGab          * Charge moved under phases 2, 3, and 4 and serial register EF
DEF #P_STATE_2     #P_PHASE_2 or #TGab          * Charge contained under phases 3 and 4 and serial register EF
DEF #P_STATE_3     #P_PHASE_3 or #TGab or #TGcd * Charge moved under phases 3, 4, and 1 and serial register EF and GH
DEF #P_STATE_4     #P_PHASE_4 or #TGab or #TGcd * Charge contained under phases 4 and 1 and serial register EF and GH
DEF #P_STATE_5     #P_PHASE_5 or #TGcd          * Charge moved under phases 4, 1, and 2 and serial register GH
DEF #P_STATE_6     #P_PHASE_6 or #TGcd          * Charge contained under phases 1 and 2 and serial register GH
DEF #P_STATE_7     #P_PHASE_7                   * Charge moved under phases 1, 2, and 3
DEF #P_STATE_8     #P_PHASE_8                   * Charge contained under phases 2 and 3, Copy of state 0 i.e. integration phase

*
*******************************
** SERIAL CLOCK BIT PATTERNS **
*******************************
* Uncomment here for four amplifier mode EFGH
DEF #S1     #S1e or #S1f or #S1g or #S1h    
DEF #S2     #S2e or #S2f or #S2g or #S2h
DEF #S3     #S3ef or #S3gh
DEF #RST    #RGe or #RGf or #RGg or #RGh
DEF #SW     #SWe or #SWf or #SWg or #SWh
DEF #DG     #DGab or #DGcd

** ELEMENTAL PATTERNS TO MOVE CHARGE ONE PIXEL TOWARDS AMPLIFIER
DEF #S_PHASE_0    #S2 or #S3
DEF #S_PHASE_1    #S2
DEF #S_PHASE_2    #S2 or #S1
DEF #S_PHASE_3    #S1
DEF #S_PHASE_4    #S1 or #S3
DEF #S_PHASE_5    #S3
DEF #S_PHASE_6    #S2 or #S3

** READ ONE PIXEL - With Binning
DEF #S_STATE_0     #S_PHASE_0 or #SW
DEF #S_STATE_1     #S_PHASE_1 or #SW or #RST 
DEF #S_STATE_2     #S_PHASE_2 or #SW
DEF #S_STATE_3     #S_PHASE_3 or #SW
DEF #S_STATE_4     #S_PHASE_4 or #SW
DEF #S_STATE_5     #S_PHASE_5 or #SW             * Sample reset
DEF #S_STATE_6     #S_PHASE_6                    * sample video

DEF #S_STATE_2T    #S_STATE_2 or #RST            * Used during parallel xfers

** INTEGRATION AND INITIALIZATION CLOCKING VECTORS
DEF #INT_0        #P_PHASE_0 or #S_PHASE_0

** PARALLEL SHIFT INTO SERIALS
DEF #P_SHIFT_0    #P_STATE_0 or #S_STATE_2T
DEF #P_SHIFT_1    #P_STATE_1 or #S_STATE_2T
DEF #P_SHIFT_2    #P_STATE_2 or #S_STATE_2T
DEF #P_SHIFT_3    #P_STATE_3 or #S_STATE_2T
DEF #P_SHIFT_4    #P_STATE_4 or #S_STATE_2T
DEF #P_SHIFT_5    #P_STATE_5 or #S_STATE_2T
DEF #P_SHIFT_6    #P_STATE_6 or #S_STATE_2T
DEF #P_SHIFT_7    #P_STATE_7 or #S_STATE_2T
DEF #P_SHIFT_8    #P_STATE_8 or #S_STATE_2T

DEF #P_XFER_0     #P_STATE_0 or #S_STATE_1
DEF #P_XFER_1     #P_STATE_0 or #S_STATE_2 or #RST
DEF #P_XFER_2     #P_STATE_0 or #S_STATE_1
DEF #P_XFER_3     #P_STATE_0 or #S_STATE_0

** PARALLEL CLEAR SEQUENCE PATTERNS
DEF #P_CLEAR_0    #P_STATE_0 or #S_PHASE_0 or #DG                * Delay  1us before => #CLEAR_1
DEF #P_CLEAR_1    #P_STATE_0 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_2
DEF #P_CLEAR_2    #P_STATE_1 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_3 
DEF #P_CLEAR_3    #P_STATE_2 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_4 
DEF #P_CLEAR_4    #P_STATE_3 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_5 
DEF #P_CLEAR_5    #P_STATE_4 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_6 
DEF #P_CLEAR_6    #P_STATE_5 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_7 
DEF #P_CLEAR_7    #P_STATE_6 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_8 
DEF #P_CLEAR_8    #P_STATE_7 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_9 
DEF #P_CLEAR_9    #P_STATE_8 or #S_PHASE_0 or #DG or #RST        * Delay 10us before => #CLEAR_10
DEF #P_CLEAR_10   #P_STATE_8 or #DG or #RST                      * All serial phases low. Delay 20us before => #CLEAR_11
DEF #P_CLEAR_11   #P_STATE_8 or #RST                             * All serial phases low. Dump gate low, Delay 20us before => #CLEAR_12
DEF #P_CLEAR_12   #P_STATE_0 or #S_PHASE_0                       * Same as INTEGRATE state.

** SERIAL CLEAR SEQUENCE PATTERNS
DEF #S_CLEAR_0    #P_STATE_0 or #S_PHASE_0 or #RST
DEF #S_CLEAR_1    #P_STATE_0 or #S_PHASE_1 or #RST
DEF #S_CLEAR_2    #P_STATE_0 or #S_PHASE_2 or #RST
DEF #S_CLEAR_3    #P_STATE_0 or #S_PHASE_3 or #RST
DEF #S_CLEAR_4    #P_STATE_0 or #S_PHASE_4 or #RST
DEF #S_CLEAR_5    #P_STATE_0 or #S_PHASE_5 or #RST
DEF #S_CLEAR_6    #P_STATE_0 or #S_PHASE_6 or #RST

** SERIAL CLOCKS - READ OUT ONE PIXEL
DEF #S_READ_0     #S_STATE_0 or #P_STATE_0
DEF #S_READ_1     #S_STATE_1 or #P_STATE_0
DEF #S_READ_2     #S_STATE_2 or #P_STATE_0
DEF #S_READ_3     #S_STATE_3 or #P_STATE_0
DEF #S_READ_4     #S_STATE_4 or #P_STATE_0
DEF #S_READ_5     #S_STATE_5 or #P_STATE_0
DEF #S_READ_6     #S_STATE_6 or #P_STATE_0

***************************************
** TIMING CONSTANTS FOR SLOW READOUT **
** These for for a 2us dwell         **
***************************************
** Sequencer clock period is 37ns
** Clock sequence delays
DEF #PAR_CLR_DLY     10       * 10us Interphase overlap for parallels for shift during parallel clear
DEF #PAR_DMP_DLY     20       * 20us dump and recombination period for serial clocks and dump gate during parallel clear 
DEF #PAR_SHFT_DLY     8       * 10us Interphase overlap for parallels for shift during readout
DEF #PAR2SER_DLY     10       * 10us Delay between last parallel clock on par to ser transfer and first serial clock edge
DEF #SER2PAR_DLY     11       * 10us Delay between last serial sequence clock  and first parallel clock edge for next transfer
DEF #SERIAL_DLY       4       * (4) Delay for serial clock, approx. 150ns [seq clk]
DEF #RESET_DLY        7       * (7) Reset pulse delay. Approx. 500ns
* DEF #CDS_DWELL      5       * (5) CDS integration time = 300ns [seq clk]
* DEF #CDS_DWELL     16       * (16) CDS integration time = 700ns [seq clk]
* DEF #CDS_DWELL     24       * (24) CDS integration time = 1us [seq clk]
DEF #CDS_DWELL       50       * (50) CDS integration time = 2us [seq clk]
* DEF #CDS_DWELL    103       * (103) CDS integration time = 4us [seq clk]
DEF #RST_HOLDOFF      5       * (5)Signal settling delay before reset dwell time
DEF #VID_HOLDOFF      0       * (0) Signal settling delay before video dwell time.
DEF #ADC_SETTLE       5       * (5) Delay for the signal to settle after last integration period and before ADC samples the result = 520ns [seq clk]
DEF #ADC_HOLD         2       * (2) Hold time for ADC acquisition before clamping down on the video processor approx. 100ns [seq clk]
                              * Shutter closing delay .. to allow shutter to fully close before reading out.
DEF #POST_INT_DLY   100       * 100ms Post integration delay, after shutter is commanded to close, before readout commences
*
**
*******************************
** LOOP REGISTER DEFINITIONS ** combined - Definitions from the MONSOON Orange system
******************************* Now mapped to the Torrent register values
DEF #ROWS          0  * 1                 * NUMBER OF ROWS TO READOUT (DET_ROWS / NPBIN)
DEF #COLS          1  * 0                 * NUMBER OF COLUMNS TO READOUT (DET_COLS / NSBIN)
DEF #ROWBIN        2  * 3                 * PARALLEL BINNING FACTOR
DEF #COLBIN        3  * 2                 * SERIAL BINNING FACTOR
DEF #ROWSKP        4  * 5                 * Number of rows to skip for ROI readout
DEF #COLSKP        5  * 4                 * Number of columns to skip for ROI readout
DEF #POST_BIN_COLS 7  * 8                 * Number of columns to skip after roi read and before overscan read
DEF #PREBINSKIP    9  * 10                 * Number of x pixels to skip before starting a binned readout. Makes for prettier images :-)
DEF #POSTSCAN_ROI  12 * 9                 * Number of post scan pixels to read after ROI area
**
**************************************************************
** JUMP FLAG DEFINITIONS - BIT POSITION IN COMMAND REGISTER ** from fred - In original MONSOON Orange format
************************************************************** Now mapped to the Torrent standard 
DEF #SYNC          1                 * Jump on sync bit high
DEF #ITC           2                 * Jump on Integration time expired
DEF #START         3                 * Jump on start exposure flag set
DEF #CCDREADOUT    4                 * STRVCTR =  1: NORMAL CYCLE - CLEAR, INTEGRATE, READOUT
DEF #CCDCLEAR      5                 * STRVCTR =  2: CLEAR ONLY
DEF #CCDREAD       6                 * STRVCTR =  4: READOUT ONLY
DEF #VCTR_4        7                 * STRVCTR =  8:
DEF #VCTR_5        8                 * STRVCTR = 16: 
DEF #VCTR_6        9                 * STRVCTR = 32: 
DEF #VCTR_7       10                 * STRVCTR = 64:
DEF #VCTR_8       11                 * STRVCTR =128:
DEF #CONT_RUN     12                 * USER BIT 1: LOOP INDEFINATELY ON TESTS
DEF #SHUTTR_ENBL  15  * 13                 * Jump on user mode bit value 2  (Shutter follows integration timer)
DEF #SBIN         13  * 14                 * Jump on user mode bit value 4  (Serial binning enable bit)
DEF #ROI          14  * 15                 * Jump on user mode bit value 8  (ROI Readout enable bit)
**
****************************
** EFR REGISTER CONSTANTS ** Used with 16-bit or 32-bit write modes 
****************************
DEF #KILL_EXPFLG    x01080001 * Cancel start exposure flag (bit 3 of Command Register) using the EFR register
DEF #PULSE_PWRSYNC  x01080002 * Pulse the power synchronization strobe for power supply synchronization
DEF #STRT_INTCNTR   x01080004 * Start the integration time counter using the EFR register
DEF #STOP_INTCNTR   x01080008 * Stop the integration time counter using the EFR register
DEF #OPEN_SHUTTER   x01080010 * Command the shutter to open
DEF #CLOSE_SHUTTER  x01080020 * Command the shutter to close
DEF #PREFLASH_ON    x01080040 * Turn the preflash signal on
DEF #PREFLASH_OFF   x01080080 * Turn off the preflash signal
DEF #PULSE_FRAME    x01080100 * Pulse the Frame Start flag to indicate a new frame
DEF #PULSE_LINE     x01080200 * Pulse the Line s Start Flag to indicate a new line beginning
DEF #READOUT_BUSY   x01080400 * Set the readout in progress flag to the DHE - Allows shutting down certain services while reading out.
DEF #READOUT_IDLE   x01080800 * Set the readout in progress flag false
DEF #SPARE_EFR_1    x01081000 * Spare toggle bit - not used in sequencer
DEF #SPARE_EFR_2    x01082000 * Spare toggle bit - not used in sequencer
DEF #SET_SYNC_LOW   x01084000 * If enabled as master DHE, set sync bit low after sync delay
DEF #SET_SYNC_HIGH  x01088000 * If enabled as master DHE, set sync signal high immediately
*
**   COMBINATION EFR COMMAND WORDS             
* DEF #OPEN_AND_START      #STRT_INTCNTR OR #OPEN_SHUTTER     * START INTEGRATION TIMER AND OPEN SHUTTER
* DEF #CLOSE_AND_STOP      #STOP_INTCNTR OR #CLOSE_SHUTTER    * CLOSE SHUTTER AND STOP INTEGRATION TIMER
* DEF #CLOSE_STOP_BUSY     #CLOSE_AND_STOP OR #READOUT_BUSY   * CLOSE SHUTTER, STOP INTEGRATION TIMER, AND SET 'READOUT IN PROGRESS' STATUS
* DEF #CLOSE_AND_START     #CLOSE_SHUTTER OR #STRT_INTCNTR    * CLOSE SHUTTER AND START INTEGRATION COUNTER - FOR DARK EXPOSURES
DEF #SHUTTER_OPEN        #OPEN_SHUTTER OR #PREFLASH_ON      * Specifically for Chiron to allow exposure meter to run
DEF #SHUTTER_CLOSE       #CLOSE_SHUTTER OR #PREFLASH_OFF    * Specifically for Chiron to allow exposure meter to run
*
** Define auxiliary poke values ..
DEF #TELEM_ON        1000     * Default value of 1000 milliseconds between telemetry data scans
DEF #TELEM_OFF       0        * To switch telemetry scans off, stuff 0 millisecond value as telemetry scan period
**
*****************************
** MODE REGISTER CONSTANTS **
*****************************
DEF #RESET            0       * Mode register constant for board reset operation - Not used in sequencer
DEF #READ             1       * Mode register constant for read operation - Not used in sequencer
DEF #WRITE16          2       * Mode register constant for 16 bit write operation - Module address contained in upper 16-bits of pattern data
DEF #WRITE32          3       * Mode register constant for 32 bit write operation - use LDA to adress module
**
*******************************
** LSR BOARD SELECT ASSIGNMENTS ** combined
*******************************
DEF #DSLCT            0
DEF #LCB              1
DEF #PSM              2
DEF #CFG              4
DEF #PIX              8
DEF #AFE             16
**
*******************************
** CFG MODULE REGISTER ADDRESSING
DEF #EFR_REG      x0108       * WRITE ALL EFR PARAMETERS TO THIS REGISTER WHEN USING 32-BIT MODE
DEF #TSTCOND_REG  x0109       * WRITE A 23-BIT ADDRESS HERE TO SEE the RESULT OF A READ OF THAT ADDRESS HERE
**
*******************************
** TEST CONDITION READ ADDRESSES
DEF #RD_STATUS x0001fffd            * read the system ststus word
**
*******************************
** AFE Sub addresses
DEF #CLKPORT      x3002       * AFE MODULE CLOCK PORT POINTER
DEF #CDSREG       x2002       * AFE MODULE CDS PORT POINTER
DEF #SEQCMD       x2003       * AFE CDS SEQUENCER TRIGGER POINTER
DEF #TELENBL      xf004       * Telelemtry read interval register
**
*******************************
** CDS state values - written as data for 16-bit mode writes to the cds register
*******************************
DEF #CDS_CTC  x20020001       * CTC strobe
DEF #CDS_INV  x20020002       * INVERT mode
DEF #CDS_NON  x20020004       * NON_INVERT mode
DEF #CDS_INT  x20020008       * Integrate
DEF #CDS_CLP  x20020010       * DC Restore Clamp
DEF #CDS_RST  x20020020       * RESET Integrator

*************************
** CDS state sequences **
*************************
*
**Normal readout mode sequence
DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
DEF #CDS01 #CDS_INV OR #CDS_CLP OR #CDS_RST
DEF #CDS02 #CDS_INV OR #CDS_RST
DEF #CDS03 #CDS_INV
DEF #CDS04 #CDS_INV OR #CDS_INT
DEF #CDS05 #CDS_INV
DEF #CDS06 #CDS_INV OR #CDS_NON
DEF #CDS07 #CDS_NON
DEF #CDS08 #CDS_NON OR #CDS_INT
DEF #CDS09 #CDS_NON
DEF #CDS10 #CDS_NON OR #CDS_CTC
DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST

**single phase readout mode sequence - just playing
* DEF #CDS00 #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_NON OR #CDS_CLP
* DEF #CDS02 #CDS_NON OR #CDS_INT
* DEF #CDS03 #CDS_NON
* DEF #CDS04 #CDS_NON OR #CDS_INT
* DEF #CDS05 #CDS_NON
* DEF #CDS06 #CDS_NON OR #CDS_INT
* DEF #CDS07 #CDS_NON
* DEF #CDS08 #CDS_NON OR #CDS_INT
* DEF #CDS09 #CDS_NON
* DEF #CDS10 #CDS_NON OR #CDS_CTC
* DEF #CDS11 #CDS_NON OR #CDS_CLP

* 
** "Buffer" - DC Restore + Common mode + Integrator held in reset readout mode sequence (i.e. test buffer and ADC)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS02 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS03 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS04 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS05 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS06 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS07 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS08 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS09 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS10 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  OR #CDS_CTC * Normal COMMON Mode rest state + CTC strobe
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state

*
** "Integrator" - DC Restore + Common Mode readout mode sequence (i.e. Integrator functions on offset signal from inverter stage + Buffer + ADC)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST
* DEF #CDS02 #CDS_INV OR #CDS_NON OR #CDS_RST
* DEF #CDS03 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS04 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS05 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS06 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS07 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS08 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS09 #CDS_INV OR #CDS_NON OR #CDS_CLP 
* DEF #CDS10 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_CTC
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP

*
** "Inverter" - DC Restore readout mode sequence (i.e. Phase inverter + Integrator functions + buffer + ADC tested)
* DEF #CDS00 #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST  * Normal COMMON Mode rest state
* DEF #CDS01 #CDS_INV OR #CDS_CLP OR #CDS_RST
* DEF #CDS02 #CDS_INV OR #CDS_CLP OR #CDS_RST
* DEF #CDS03 #CDS_INV OR #CDS_CLP
* DEF #CDS04 #CDS_INV OR #CDS_CLP OR #CDS_INT
* DEF #CDS05 #CDS_INV OR #CDS_CLP
* DEF #CDS06 #CDS_INV OR #CDS_CLP OR #CDS_NON
* DEF #CDS07 #CDS_NON OR #CDS_CLP
* DEF #CDS08 #CDS_NON OR #CDS_CLP OR #CDS_INT
* DEF #CDS09 #CDS_NON OR #CDS_CLP
* DEF #CDS10 #CDS_NON OR #CDS_CLP OR #CDS_CTC
* DEF #CDS11 #CDS_INV OR #CDS_NON OR #CDS_CLP

** AFE CDS SEQUENCER COMMAND TRIGGER VECTORS
DEF #CDS_NORMAL_READ    x010A00
DEF #CDS_CLMP_READ      
DEF #CDS_CLMP_RST_READ  
**
********************************
** PATTERN MEMORY DEFINITIONS ** combined
********************************
** PATTERN MEMORY SEGMENTATION MAP
DEF #PATSEG_00      x0000             * Pattern memory origin - Initialization stuff
DEF #PATSEG_CLR     x0020             * Pattern memory origin - CCD CLEAR VECTORS
DEF #PATSEG_INIT    x0040             * Pattern memory origin - CCD READ INITIALIZATION
DEF #PATSEG_PAR     x0060             * Pattern memory origin - CCD PARALLEL SHIFT VECTORS
DEF #PATSEG_SER     x0080             * Pattern memory origin - CCD SERIAL SHIFT VECTORS
DEF #PATSEG_BIN     x00A0             * Pattern memory origin - CCD SERIAL BINNING VECTORS
DEF #PATSEG_CDS     x0100             * Pattern memory origin - CDS control vectors

ORG PAT #PATSEG_00                    ********** GENERAL CONSTANTS KEPT IN PAT MEM ********* 
PAT #KILL_EXPFLG     #KILL_EXPFLG     ** We can use the same label name here because they are in different contexts
PAT #PULSE_PWRSYNC   #PULSE_PWRSYNC   ** the definition earlier points to a data value. the label here points to
PAT #STRT_INTCNTR    #STRT_INTCNTR    ** a pattern memory address
PAT #STOP_INTCNTR    #STOP_INTCNTR    ********************
PAT #OPEN_SHUTTER    #OPEN_SHUTTER    
PAT #CLOSE_SHUTTER   #CLOSE_SHUTTER   
PAT #PREFLASH_ON     #PREFLASH_ON     
PAT #PREFLASH_OFF    #PREFLASH_OFF    
PAT #PULSE_FRAME     #PULSE_FRAME     
PAT #PULSE_LINE      #PULSE_LINE      
PAT #READOUT_BUSY    #READOUT_BUSY    
PAT #READOUT_IDLE    #READOUT_IDLE    
PAT #SET_SYNC_LOW    #SET_SYNC_LOW    
PAT #SET_SYNC_HIGH   #SET_SYNC_HIGH  
* PAT #OPEN_AND_START  #OPEN_AND_START
* PAT #CLOSE_AND_STOP  #CLOSE_AND_STOP
* PAT #CLOSE_STOP_BUSY #CLOSE_STOP_BUSY
* PAT #CLOSE_AND_START #CLOSE_AND_START
PAT #SHUTTER_OPEN    #SHUTTER_OPEN
PAT #SHUTTER_CLOSE   #SHUTTER_CLOSE
                  
ORG PAT #PATSEG_CLR                 * ********* CCD PARALLEL (SPLIT) CLEAR VECTORS ********* 
PAT #P_CLEAR_0     #P_CLEAR_0
PAT #P_CLEAR_1     #P_CLEAR_1 
PAT #P_CLEAR_2     #P_CLEAR_2 
PAT #P_CLEAR_3     #P_CLEAR_3 
PAT #P_CLEAR_4     #P_CLEAR_4 
PAT #P_CLEAR_5     #P_CLEAR_5 
PAT #P_CLEAR_6     #P_CLEAR_6 
PAT #P_CLEAR_7     #P_CLEAR_7 
PAT #P_CLEAR_8     #P_CLEAR_8 
PAT #P_CLEAR_9     #P_CLEAR_9 
PAT #P_CLEAR_10    #P_CLEAR_10
PAT #P_CLEAR_11    #P_CLEAR_11
PAT #P_CLEAR_12    #P_CLEAR_12

PAT #S_CLEAR_0     #S_CLEAR_0
PAT #S_CLEAR_1     #S_CLEAR_1
PAT #S_CLEAR_2     #S_CLEAR_2
PAT #S_CLEAR_3     #S_CLEAR_3
PAT #S_CLEAR_4     #S_CLEAR_4
PAT #S_CLEAR_5     #S_CLEAR_5
PAT #S_CLEAR_6     #S_CLEAR_6

ORG PAT #PATSEG_INIT                  * ********* CCD INTEGRATION AND READ INITIALIZATION ********* 
PAT #INT_0          #INT_0
PAT #P_XFER_0       #P_XFER_0         * SERIAL CLCK SETUP BEFORE PARALLEL TRANSFER
PAT #P_XFER_1       #P_XFER_1
PAT #P_XFER_2       #P_XFER_2         * SERIAL CLCK SETUP AFTER PARALLEL TRANSFER
PAT #P_XFER_3       #P_XFER_3

ORG PAT #PATSEG_PAR                   * ********* CCD PARALLEL (SPLIT) SHIFT VECTORS ********* 
PAT #P_SHIFT_0     #P_SHIFT_0
PAT #P_SHIFT_1     #P_SHIFT_1
PAT #P_SHIFT_2     #P_SHIFT_2
PAT #P_SHIFT_3     #P_SHIFT_3
PAT #P_SHIFT_4     #P_SHIFT_4
PAT #P_SHIFT_5     #P_SHIFT_5
PAT #P_SHIFT_6     #P_SHIFT_6
PAT #P_SHIFT_7     #P_SHIFT_7
PAT #P_SHIFT_8     #P_SHIFT_8

ORG PAT #PATSEG_SER                   * ********* CCD SERIAL (SPLIT) SHIFT (BOTH READ & IDLE) VECTORS ********* 
PAT #S_READ_0      #S_READ_0
PAT #S_READ_1      #S_READ_1
PAT #S_READ_2      #S_READ_2
PAT #S_READ_3      #S_READ_3
PAT #S_READ_4      #S_READ_4
PAT #S_READ_5      #S_READ_5
PAT #S_READ_6      #S_READ_6

ORG PAT #PATSEG_CDS      ** CDS Control vectors ********* 
PAT #CDS00         #CDS00
PAT #CDS01         #CDS01
PAT #CDS02         #CDS02
PAT #CDS03         #CDS03
PAT #CDS04         #CDS04
PAT #CDS05         #CDS05
PAT #CDS06         #CDS06
PAT #CDS07         #CDS07
PAT #CDS08         #CDS08
PAT #CDS09         #CDS09
PAT #CDS10         #CDS10
PAT #CDS11         #CDS11
PAT #TELEM_ON      #TELEM_ON
PAT #TELEM_OFF     #TELEM_OFF
*
**
**************************************
** PROGRAM CODE SEGMENT STARTS HERE **
**************************************
** EXECUTIVE ROUTINE
#INIT NOP                        * Sequencer opcode pipeline problem. Use NOP delay to fill pipeline.
      LMR #WRITE32               * SET 32 BIT WRITE MODE FOR EVERYTHING
      CAL #INTEG                 * ESTABLISH THE INITIAL CONDITIONS
#EXEC JCB #GOBABY #START         * IF START EXPOSURE FLAG SET
      CAL #P_CLR                 * DO PARALLEL CLEAR CYCLES WHILE IN IDLE
      JMP #EXEC                  * ELSE JUMP BACK TO EXEC
#GOBABY
      JCB #JR_CLEAR #CCDCLEAR     * DO JUST CLEAR
      LDA #EFR_REG
      LPP #KILL_EXPFLG            * KILL THE START EXP FLAG PATTERN AFTER CLEAR SO WE CAN USE THIS FLAG TO DETECT THE END OF THE CLEAR OPERATION
      LSR #CFG                    * WRITE TO THE EFR
      JCB #JR_RDOUT #CCDREADOUT   * DO INTEGRATE - READOUT OPERATION
      JCB #JR_READ #CCDREAD       * DO JUST READOUT
      JMP #EXEC                   * NO FLAGS SET, GO HOME.
 
** EXECUTIVE LEVEL CALLS
** NORMAL READOUT FROM START VECTOR = 1
#JR_RDOUT 
*      CAL #CLEAR                 * !!!! Remove this call for use with Chiron - to enable accurate timing with exposure meter.
      LDA #EFR_REG
      LPP #STRT_INTCNTR
      LSR #CFG                   * START INTEGRATION TIMER RUNNING
      CAL #OPEN_SHUT             * open the shutter
#RDOUT_1
      JCB #RDOUT_2 #ITC          * WAIT HERE FOR INTEGRATION TIMER TO FINISH
      JMP #RDOUT_1
#RDOUT_2
      CAL #CLOSE_SHUT            * Start the shutter closing
      LDA #EFR_REG
      LPP #STOP_INTCNTR
      LSR #CFG                   * STOP INTEGRATION TIMER
      LDA #TELENBL
      LPP #TELEM_OFF
      LSR #AFE                   * Switch off any telemetry scans while we read out ... bug in firmware that should do this anyway !
      DMS #POST_INT_DLY          * Delay for shutter to reach closed position - Added version 2.1
      JCB #RDOUT_3 #ROI          * CALL A DIFFERENT ROUTINE FOR ROI READOUT    
      CAL #READ                  * READOUT WHEN ITC HAS REACHED REQUIRED TIME
      JMP #RDOUT_4
      
#RDOUT_3
      CAL #READ_ROI
      
#RDOUT_4
      LDA #TELENBL
      LPP #TELEM_ON
      LSR #AFE                   * Switch on the telemetry scans while idle ... bug in firmware that should do this anyway !
      JMP #EXEC                  * CALL IT A DAY

** JUST CLEAR FROM START VECTOR = 2
#JR_CLEAR 
      CAL #CLEAR
      JCB #JR_CLEAR #CONT_RUN    * DO IT ALL AGAIN IF TOLD TO DO SO
      LDA #EFR_REG
      LPP #KILL_EXPFLG            * KILL THE START EXP FLAG PATTERN AFTER CLEAR SO WE CAN USE THIS FLAG TO DETECT THE END OF THE CLEAR OPERATION
      LSR #CFG                    * WRITE TO THE EFR
      JMP #EXEC
         
** JUST READOUT FROM START VECTOR = 4
#JR_READ 
      JCB #JR_RDX #ROI           * CALL A DIFFERENT ROUTINE FOR ROI READOUT    
      CAL #READ
      JCB #JR_READ #CONT_RUN     * DO IT ALL AGAIN IF TOLD TO DO SO
      JMP #EXEC         
#JR_RDX 
      CAL #READ_ROI
      JCB #JR_READ #CONT_RUN    * DO IT ALL AGAIN IF TOLD TO DO SO
      JMP #EXEC         
      
*****************************
** REPEATED SEQUENCE LOOPS ** 
*****************************
#INTEG
      LDA #CLKPORT
      LPP #INT_0                  * Set P2 + P3 + S1 + S2
      LSR #AFE                    * Establish the clock phasing during integration
      LDA #CDSREG
      LPP #CDS00
      LSR #AFE                    * Set rest mode to the CDS circuits
      LDA #EFR_REG
      LPP #READOUT_IDLE
      LSR #CFG                    * reset readout busy - idle state so allow telemetry, cryostat heater, etc.
      RET
         
#CLEAR
      LRB #ROWS                   * Start of row loop
      LRB #ROWBIN                 * multiply by the binning factor if set
      CAL #P_CLR                  * RUN THE CLEAR SEQUENCE FOR PARALLELS
      LPE                         * End row binning loop
      LPE                         * End of row loop
      CAL #INTEG                  * SET EXPOSURE INTEGRATION PATTERN
      RET

#RINIT  
      LDA #EFR_REG
      LPP #READOUT_BUSY
      LSR #CFG                * set readout busy
      LPP #PULSE_FRAME
      LSR #CFG                * set the new frame flag for the acquisition firmware
      RET

#READ 
      CAL #RINIT                  * initialize for read   
      CAL #SER_CLEAR              * CLEAR THE SERIAL REGISTER
      DUS #SER2PAR_DLY            * Delay timing between end of serial sequence and first parallel clock movement for next transfer 
      LRB #ROWS                   * Start row loop
      CAL #PRE_PAR_CLK            * SETUP THE SERIALS FOR A TRANSFER
      LRB #ROWBIN                 * Start row binning loop
      CAL #PAR_SHIFT              * DO ROW XFER
      LPE                         * End of row binning loop 
      DUS #PAR2SER_DLY            * Delay timing between end of parallel xfer clocks and first serial clock movement
      CAL #POST_PAR_CLK           * SETUP THE SERIALS TO DO SERIAL XFERS
      LDA #EFR_REG
      LPP #PULSE_LINE
      LSR #CFG                    * indicate a new row to the acquisition hardware
      LRB #COLS                   * Start of column loop      
      CAL #SER_READ               * DO COLUMN READ FOR N BINS
      LPE                         * END OF COLUMN LOOP
      DUS #SER2PAR_DLY            * Delay timing between end of serial sequence and first parallel clock movement for next transfer 
      LPE                         * END OF ROW LOOP
      CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
      RET
      
#READ_ROI
      CAL #RINIT                  * initialize for read   
      LRB #ROWSKP                 * LOOP FOR THE ROW SKIP COUNT
      CAL #P_CLR                  * CLEAR THE REQUIRED NUMBER OF ROWS  
      LPE
      CAL #SER_CLEAR              * CLEAR THE SERIAL REGISTER
      DUS #SER2PAR_DLY            * Delay timing between end of serial sequence and first parallel clock movement for next transfer 
      LRB #ROWS                   * LOOP FOR N ROWS
      CAL #PRE_PAR_CLK            * SETUP THE SERIALS FOR A TRANSFER
      LRB #ROWBIN                 * Start row binning loop      
      CAL #PAR_SHIFT              * DO ROW XFER
      LPE                         * End of row binning loop
      DUS #PAR2SER_DLY            * Delay timing between end of parallel xfer clocks and first serial clock movement
      CAL #POST_PAR_CLK           * SETUP THE SERIALS TO DO SERIAL XFERS
      LDA #EFR_REG
      LPP #PULSE_LINE
      LSR #CFG                    * indicate a new row to the acquisition hardware
      LRB #COLSKP                 * LOOP FOR THE SERIAL PIXEL SKIP COUNT 
      CAL #S_CLR                  * CLEAR THE SERIAL PIXEL OUT
      LPE
      LRB #COLS                   * LOOP FOR N COLS      
      CAL #SER_READ               * DO COLUMN READ FOR N BINS
      LPE                         * END OF COLUMN LOOP
      LRB #POST_BIN_COLS          * Now skip over to the overscan region
      CAL #S_CLR                  * CLEAR THE SERIAL PIXEL OUT
      LPE
      LRB #POSTSCAN_ROI           * Now readout an overscan region
      CAL #SER_READ               * DO COLUMN READ FOR N BINS      
      LPE      
      DUS #SER2PAR_DLY            * Delay timing between end of serial sequence and first parallel clock movement for next transfer 
      LPE                         * END OF ROW LOOP
      CAL #INTEG                  * PUT THE DETECTOR BACK INTO INTEGRATE MODE
      RET
      
#PRE_PAR_CLK
      LDA #CLKPORT
      LPP #P_XFER_0
      LSR #AFE                 * Set P2 + P3 + S2 + RST
      DSC #SERIAL_DLY
      LPP #P_XFER_1
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST
      DSC #SERIAL_DLY     
      RET

#POST_PAR_CLK
      LDA #CLKPORT
      LPP #P_XFER_2
      LSR #AFE                 * Set P2 + P3 + S2 + RST
      DSC #SERIAL_DLY
      LPP #P_XFER_3
      LSR #AFE                 * Set P2 + P3 + S2 + S3 + RST
      DSC #SERIAL_DLY     
      RET
      
#SER_CLEAR
      LRB #COLS                * Start of column loop           
      LRB #COLBIN              * Multiply by th ebinning factor if set
      CAL #S_CLR               * RUN THE CLEAR SEQUENCE FOR SERIAL PIXELS
      LPE                      * End of column binning loop
      LPE                      * End of column loop
      CAL #INTEG               * SET EXPOSURE INTEGRATION PATTERN
      RET

***********************************************
** LOW LEVEL SEQUENCE PRIMATIVES WITH TIMING **
***********************************************
#SER_READ                       * read one pixel: shift, process, A/D convert, xmit
        LDA #CDSREG             * Comes in with S1
        LPP #CDS00              * #CDS_INV OR #CDS_NON OR #CDS_CLP OR #CDS_RST 
        LSR #AFE
        LDA #CLKPORT
        LRB #ColBin             * Load serial binning
       
        LPP #S_READ_0           *  S1  S2
        LSR #AFE 
        DSC #SERIAL_DLY         * one serial delay time 

        LPP #S_READ_1           *      S2      RG
        LSR #AFE 
        DSC #RESET_DLY          * reset clock time 

        LPP #S_READ_2           *      S2  S3
        LSR #AFE 
        DSC #SERIAL_DLY         * one serial delay time 

        LPP #S_READ_3           * S3 SW
        LSR #AFE 
        DSC #SERIAL_DLY         * one serial delay time 

        LPP #S_READ_4           * S1      S3  SW
        LSR #AFE 
        DSC #SERIAL_DLY         * one serial delay time 

        LPP #S_READ_5           * S1         SW
        LSR #AFE 
        DSC #SERIAL_DLY         * one serial delay time 
        
        LPE                     * end of serial binning loop
                                * Charge is under SW        

        LDA #CDSREG
        LPP #CDS01              * #CDS_INV OR #CDS_CLP OR #CDS_RST
        LSR #AFE
        LPP #CDS02              * #CDS_INV OR #CDS_RST
        LSR #AFE
        LPP #CDS03              * #CDS_INV
        LSR #AFE

        DSC #RST_HOLDOFF        * Wait for reset signal to settle

        LPP #CDS04              * #CDS_INV OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL          * integrate on the reset level
        
        LPP #CDS05              * #CDS_INV
        LSR #AFE
        LPP #CDS06              * #CDS_INV OR #CDS_NON
        LSR #AFE

        LDA #CLKPORT
        LPP #S_READ_6           * SW taken low - charge dumped into sense node
        LSR #AFE 

        LDA #CDSREG
        LPP #CDS07              * #CDS_NON
        LSR #AFE
        
        DSC #VID_HOLDOFF        * Wait for video signal to settle
        
        LPP #CDS08              * #CDS_NON OR #CDS_INT
        LSR #AFE

        DSC #CDS_DWELL          * integrate on the video level

        LPP #CDS09              * #CDS_NON
        LSR #AFE
        DSC #ADC_SETTLE         * Wait for the signal to settle on the ADC
        LPP #CDS10              * #CDS_NON OR #CDS_CTC
        LSR #AFE
        DSC #ADC_HOLD
        LPP #CDS11              * #CDS_NON
        LSR #AFE
        LDA #EFR_REG
        LPP #PULSE_PWRSYNC
        NOP        
*        LSR #CFG                * synchronize the power supply here           
        RET
     
* PARALLEL TRANSFERS      
#PAR_SHIFT
      LDA #CLKPORT
      LPP #P_SHIFT_0
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_1
      LSR #AFE                 * Set P2 + P3 + P4 + TGab + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_2
      LSR #AFE                 * Set P3 + P4 + TGab + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_3
      LSR #AFE                 * Set P1 + P3 + P4 + TGab + TGcd + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_4
      LSR #AFE                 * Set P1 + P4 + TGab + TGcd + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_5
      LSR #AFE                 * Set P1 + P2 + P4 + TGd + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_6
      LSR #AFE                 * Set P1 + P2 + TGcd + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_7
      LSR #AFE                 * Set P1 + P2 + P3 + S1 + S2 + RST 
      DUS #PAR_SHFT_DLY           * Delay 10 us
      LPP #P_SHIFT_8
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      RET

#P_CLR  
      LDA #CLKPORT 
      LPP #P_CLEAR_0
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + DG + RST 
      DUS 1                       * Delay 1 us before starting the parallel transfer
      LPP #P_CLEAR_1
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + DG + RST
      DUS #PAR_CLR_DLY            * Delay 10 us before going to CLEAR_2
      LPP #P_CLEAR_2
      LSR #AFE                 * Set P2 + P3 + P4 + S1 + S2 + DG + RST  
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_3
      LSR #AFE                 * Set P3 + P4 + S1 + S2 + DG + RST 
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_4
      LSR #AFE                 * Set P3 + P4 + P1 + S1 + S2 + DG + RST
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_5
      LSR #AFE                 * Set P4 + P1 + S1 + S2 + DG + RST  
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_6
      LSR #AFE                 * Set P4 + P1 + P2 + S1 + S2 + DG + RST 
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_7
      LSR #AFE                 * Set P1 + P2 + S1 + S2 + DG + RST
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_8
      LSR #AFE                 * Set P1 + P2 + P3 + S1 + S2 + DG + RST 
      DUS #PAR_CLR_DLY            * Delay 10 us
      LPP #P_CLEAR_9
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + DG + RST  
      DUS #PAR_CLR_DLY            * Delay 10 us with all serails low
      LPP #P_CLEAR_10
      LSR #AFE                 * Set P2 + P3 + DG + RST 
      DUS #PAR_DMP_DLY            * Pulse the Dump gate for 20us to recombine charge to substrate
      LPP #P_CLEAR_11
      LSR #AFE                 * Set P2 + P3 + RST
      DUS #PAR_CLR_DLY            * Pulse the Dump gate for 10us to go back to idle state
      LPP #P_CLEAR_12
      LSR #AFE                 * Set P2 + P3 + S1 + S2
      RET

#S_CLR
      LDA #CLKPORT 
      LPP #S_CLEAR_0
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_1
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_2
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_3
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_4
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_5
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      LPP #S_CLEAR_6
      LSR #AFE                 * Set P2 + P3 + S1 + S2 + RST 
      RET
      
********************************
** Some utility code snippets **
********************************

#OPEN_SHUT
      LDA #EFR_REG
      LPP #SHUTTER_OPEN
      LSR #CFG                   * Write the appropriate word to the DOP register to open the shutter
      RET
   
#CLOSE_SHUT
      LDA #EFR_REG
      LPP #SHUTTER_CLOSE 
      LSR #CFG                   * Write the appropriate word to the DOP register to close the shutter
      RET
      
* END OF PROGRAM SEGMENT
