# Setup Mode File for Chiron spectrograph using E2V CCD231-84 Detector
# Version n+1 - Peter Moore    - 03/03/2011 - Adapted from _fred system
# This version for high speed readout at 400KPix/channel/sec    
#
# Attribute Name  = Value   #Comments
# Preamble - Kill the sequencer run flag if set
SeqEnable=0
#
## Setup some software defaults
arrayID=08442-22-01
arrayType=E2V_CCD231-84
mosaicRows=1
mosaicCols=1
PreScan=0       # 50 prescan in x 
xPostScan=0     # 50 postscan in x
yPostScan=0     # 0 post scan in y
numOutputs=2    # 2 output channels
outputCfg=0     # output configuration (0=AB or CD, 1=AD or BC, 2=AC or BD)
pxlRows=4112
pxlCols=4196
xStart[0]=1
yStart[0]=1
finDataType=32   # DHS_UINT four bytes 32 bits
frmsPerRdOut=1   # number of readout for one simple exposure
captureMode=1    # capture mode 0=NORMAL, 1=SEP, 2=BLOCKREAD
expMode=0        # mode of the exposure
processMode=0    # Processing mode -- Switch off the telemetry collection
expVector=1      # 1=Normal Clear,Integ,Readout, 2=Clear only, 4=Readout only
asyncVector=0    # Currently not used.
rawPxlSize=4     # number of Bytes in a raw pixel value
finPxlSize=4     # number of Bytes in a finished pixel value
avNameSIze=32    # Max character length of attribute name
avValueSize=32   # Max character length of attribute value
avCommentSize=64 # Max character length of attribute comment
#
#
## Clocks voltages setup for E2V
# Serial clocks
S1e_LowDac=1.0
S1e_HiDac=11.0
S1f_LowDac=1.0
S1f_HiDac=11.0
S1g_LowDac=1.0
S1g_HiDac=11.0
S1h_LowDac=1.0
S1h_HiDac=11.0
S2e_LowDac=1.0
S2e_HiDac=11.0
S2f_LowDac=1.0
S2f_HiDac=11.0
S2g_LowDac=1.0
S2g_HiDac=11.0
S2h_LowDac=1.0
S2h_HiDac=11.0
S3ef_LowDac=1.0
S3ef_HiDac=11.0
S3gh_LowDac=1.0
S3gh_HiDac=11.0
# Summing wells
SWe_LowDac=1.0
SWe_HiDac=11.0
SWf_LowDac=1.0
SWf_HiDac=11.0
SWg_LowDac=1.0
SWg_HiDac=11.0
SWh_LowDac=1.0
SWh_HiDac=11.0
# Transfer gates
TGab_LowDac=0.0
TGab_HiDac=10.0
TGcd_LowDac=0.0
TGcd_HiDac=10.0
# Parallels
P1ab_LowDac=0.0
P1ab_HiDac=10.0
P1cd_LowDac=0.0
P1cd_HiDac=10.0
P2ab_LowDac=0.0
P2ab_HiDac=10.0
P2cd_LowDac=0.0
P2cd_HiDac=10.0
P3ab_LowDac=0.0
P3ab_HiDac=10.0
P3cd_LowDac=0.0
P3cd_HiDac=10.0
P4ab_LowDac=0.0
P4ab_HiDac=10.0
P4cd_LowDac=0.0
P4cd_HiDac=10.0
# Reset gates
RGe_LowDac=0.0
RGe_HiDac=12.0
RGf_LowDac=0.0
RGf_HiDac=12.0
RGg_LowDac=0.0
RGg_HiDac=12.0
RGh_LowDac=0.0
RGh_HiDac=12.0
# Dump gates
DGcd_LowDac=0.0
DGcd_HiDac=12.0
DGab_LowDac=0.0
DGab_HiDac=12.0

## CCD board Telemetry mode setup
CCD_HVBIASTELMODE[]=12      # Internal Clock Mode, +/-10V full scale.
#
## Bias voltages setup
HVBiasDacs[]=0.0          # Set all dacs to zero
HVBiasDacs[8]=2047        # Set all bipolar dacs to mid scale
HVBiasDacs[9]=2047
HVBiasDacs[10]=2047
HVBiasDacs[11]=2047
HVBiasDacs[12]=2047
HVBiasDacs[13]=2047
HVBiasDacs[14]=2047
HVBiasDacs[15]=2047

## Bias voltages setup for E2V
ODe_Dac=26.0
ODf_Dac=26.0
ODg_Dac=24.9
ODh_Dac=24.9
OGe_Dac=2.0
OGf_Dac=2.0
OGg_Dac=2.0
OGh_Dac=2.0
RDe_Dac=17.0
RDf_Dac=17.0
RDg_Dac=17.0
RDh_Dac=17.0
DDcd_Dac=24.0
DDab_Dac=24.0

#
## Exposure Attribute setup
filename=Chiron    # Default file name prefix
intTime=0          # Default to 0 seconds
#
## Sequencer attributes
cols=2098          # 4096 image cols + 50 Prescan + 50 Postscan
rows=4112          # 1024 + 1032 x 2 image rows per output
rowBin=1
colBin=1

#
SeqClkDiv=0
contRun=0
ShuttrEnbl=1           # Enable the shutter drive
#
## Offset voltages setup
CCD_VOFFSETS[]=2048    # Set all ADC offsets safe at zero offset
VidOffset1=71000       # Empirically derived offsets for each video channel
VidOffset3=71070       # to level them to approx. 2500 ADU
VidOffset5=70740
VidOffset7=70931
#
## Video channels setup (warp)
CCD_ADCCFG[]=2
#
## Set up 16-bit (17:21) pixel data
DataWidth=2
#
## Initialize some of the ports
CdsReg=0x136      # Kill all CDS activity
DopReg=0          # Write zeros to DOP port
#
## Acquisition Board setup ##
Redirect[0]=7
Redirect[1]=5     # sample the even channels for high speed readout - Not yet !!
Redirect[2]=0
Redirect[3]=0
Redirect[4]=0
Redirect[5]=0
Redirect[6]=0
Redirect[7]=0
Redirect[8]=0
Redirect[9]=0
Redirect[10]=0
Redirect[11]=0
Redirect[12]=0
Redirect[13]=0
Redirect[14]=0
XfrCount=2     # Two pixels to xfer for each CTC
DataWidth=3    # Select 18 bit data mode Bits 17:0
AdcMode=0      # Slow mode (i.e data xfer after conversion)
PipePriority=0 # Only one acquisition board so highest priority
PipeEnbl=1     # Enable pipeline data writes
#
## Setup the voltages and enable to array
CdsEnbl=1           # Enable the CDS function
CpldLedEnable=1     # Switch off the CPLD LEDs
clkD5LedCntrl=1     # Switch off the CLK lights ...
#
## Diagnostics ...
# mcbMpuSeqBusReq=1   # Led on mcb for sequencer bus req
# ccdBrdSel=0         # Ditto for CCD board
# ccdCdsLoad=1        # light the sky when cds load strobe present
# labSystem=1         # Kill long star date file names
## CB Board auxiliary functions
clkP2MuxSlct=18      # Look at SWe
clkP1MuxSlct=16      # Look at RGe
